#ifndef __NEOCCA_H__
#define __NEOCCA_H__

/** compile time version numbers in dotted string form. */
#define NEOCCA_VERSION "0.2.8"
/** compile time major version number */
#define NEOCCA_VERSION_MAJOR 0
/** compile time minor version number */
#define NEOCCA_VERSION_MINOR 2
/** compile time patch version number */
#define NEOCCA_VERSION_PATCH 8

#include <vector>
#include <string>
#include <complex>
#include <exception>
#include <boost/cstdint.hpp>
#include <boost/shared_ptr.hpp>

/** This is neoclassic -- the ::classic::gov::cca headers updated
 * to be similar to the official CCA specification in sidl
 * while retaining the low impact on component coding that
 * the classic specification had..
 * The namespace for it is neo::cca to reduce the
 * repetitive stress injuries on cca code writers.
 *
 * The goal here is two-part:
 * 1. Minimize the intrusion of our chosen memory technique
 *    (using boost shared_ptr.hpp) on science component writers.
 *    That is, the most common user objects (Component, Port,
 *    Services) are simply virtual interfaces and have strict
 *    life-cycle rules so that reference counting is not needed
 *    and not inflicted on the component-writer's code.
 *
 * 2. Make a standards-based ( STL+boost )
 *    memory-safe interface for folks who want a simple
 *       "interfaces only"
 *    C++ binding for CCA components instead of using a
 *    heavyweight, multilanguage middleware. There are
 *    many exotic little objects in framework-related
 *    service Ports that must be reference counted.
 *
 * We have chosen the boost::shared_ptr as the least
 * annoying solution in the open-standards c++ world.
 * Component writers accessing these (mostly little used)
 * framework services may encounter shared_ptr. They
 * should consult doc/README-memory.txt for tips on
 * sensible usage.
 *
 * Largely the names used are the same as in the classic version to
 * reduce editing needed when converting to neo from classic.
 * In a most cases, return codes nobody was checking anyway
 * have been converted to exception throwing per the SIDL spec.
 * A concrete exception class implementing the std::exception
 * is thrown.
 * A component may not support both classic and neo
 * specifications in the same compilation, due to c++ overloading rules.
 * It's possible, with appropriate #ifdef, to build
 * either kind of component from the same source files,
 * but why bother?
 */
namespace neo {

/** 
 * <p>This binding lives in C++ namespace ::neo::cca .</p>
 */
namespace cca {

/* First some forward declarations. no doc comments at this point. */
class Version;
class Port;
class Component; 
class Services;
class TypeMap;
class ComponentID;
class ConnectionID;
class ComponentRelease; 

/* Then reserve and make plain the usual C/C++ conventions for
 * typedefing raw pointers. Component writers should not be
 * using these. Just do WYSIWIG coding with explicit 'Foo *' if
 * you must. These come in very handy, however, for framework
 * writers.
 */
typedef Port * Port_p;
typedef Port * Port_ptr;
typedef Component * Component_p; 
typedef Component * Component_ptr; 
typedef Services * Services_p;
typedef Services * Services_ptr;
  /* The component writer has no business putting hands
   * directly on any of the following six, as they are
   * always inside boost wrappings.
   */
typedef TypeMap * TypeMap_p;
typedef TypeMap * TypeMap_ptr;
typedef ComponentID * ComponentID_p;
typedef ComponentID * ComponentID_ptr;
typedef ConnectionID * ConnectionID_p;
typedef ConnectionID * ConnectionID_ptr;

/* We will see elsewhere in the headers, and with doc comments:
// typedef boost::shared_ptr<TypeMap> TypeMap_shared;
// typedef boost::shared_ptr<ComponentID> ComponentID_shared;
// typedef boost::shared_ptr<ConnectionID> ConnectionID_shared;
*/

}  // end namespace cca
}  // end namespace neo


//===================================================================

#include "neobase/Version.hh"
#include "neobase/Port.hh"

/* include the exception, typemap, componentid and enum definitions
 * that are not directly relevant to the cca design pattern
 * but for the c++ compiler must come first.
 */
#include "neoaux.hh"

#include "neobase/Component.hh"
#include "neobase/Services.hh"
#include "neobase/AbstractFramework.hh"

#endif //__NEOCCA_H_ 
